@@ -169,24 +169,74 @@ static guint signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (XfpmPower, xfpm_power, G_TYPE_OBJECT)
 
+
+/* This checks if consolekit returns TRUE for either suspend or
+ * hibernate showing support. This means that ConsoleKit2 is running
+ * (and the system is capable of those actions).
+ */
+static gboolean
+check_for_consolekit2 (XfpmPower *power)
+{
+    XfpmConsoleKit *console;
+    gboolean can_suspend, can_hibernate;
+
+    g_return_val_if_fail (XFPM_IS_POWER (power), FALSE);
+
+    if (power->priv->console == NULL)
+	return FALSE;
+
+    console = power->priv->console;
+
+    g_object_get (G_OBJECT (console),
+		  "can-suspend", &can_suspend,
+		  NULL);
+    g_object_get (G_OBJECT (console),
+		  "can-hibernate", &can_hibernate,
+		  NULL);
+
+    /* ConsoleKit2 supports suspend and hibernate */
+    if (can_suspend || can_hibernate)
+    {
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
 #ifdef ENABLE_POLKIT
 static void
 xfpm_power_check_polkit_auth (XfpmPower *power)
 {
-    const char *suspend, *hibernate;
+    const char *suspend = NULL, *hibernate = NULL;
     if (LOGIND_RUNNING())
     {
+	XFPM_DEBUG ("using logind suspend backend");
 	suspend   = POLKIT_AUTH_SUSPEND_LOGIND;
 	hibernate = POLKIT_AUTH_HIBERNATE_LOGIND;
     }
     else
     {
 #if !UP_CHECK_VERSION(0, 99, 0)
+	XFPM_DEBUG ("using upower suspend backend");
 	suspend   = POLKIT_AUTH_SUSPEND_UPOWER;
 	hibernate = POLKIT_AUTH_HIBERNATE_UPOWER;
 #else
-	suspend   = POLKIT_AUTH_SUSPEND_XFPM;
-	hibernate = POLKIT_AUTH_HIBERNATE_XFPM;
+	if (power->priv->console != NULL)
+	{
+	    /* ConsoleKit2 supports suspend and hibernate */
+	    if (check_for_consolekit2 (power))
+	    {
+		XFPM_DEBUG ("using consolekit2 suspend backend");
+		suspend   = POLKIT_AUTH_SUSPEND_CONSOLEKIT2;
+		hibernate = POLKIT_AUTH_HIBERNATE_CONSOLEKIT2;
+	    }
+	    else
+	    {
+		XFPM_DEBUG ("using xfpm internal suspend backend");
+		suspend   = POLKIT_AUTH_SUSPEND_XFPM;
+		hibernate = POLKIT_AUTH_HIBERNATE_XFPM;
+	    }
+	}
 #endif
     }
     power->priv->auth_suspend = xfpm_polkit_check_auth (power->priv->polkit,
@@ -269,8 +319,20 @@ xfpm_power_get_properties (XfpmPower *power)
     }
     else
     {
-	power->priv->can_suspend   = xfpm_suspend_can_suspend ();
-        power->priv->can_hibernate = xfpm_suspend_can_hibernate ();
+	if (check_for_consolekit2 (power))
+	{
+	    g_object_get (G_OBJECT (power->priv->console),
+			  "can-suspend", &power->priv->can_suspend,
+			  NULL);
+	    g_object_get (G_OBJECT (power->priv->console),
+			  "can-hibernate", &power->priv->can_hibernate,
+			  NULL);
+	}
+	else
+	{
+	    power->priv->can_suspend   = xfpm_suspend_can_suspend ();
+	    power->priv->can_hibernate = xfpm_suspend_can_hibernate ();
+	}
     }
 #endif
     g_object_get (power->priv->upower,
@@ -385,6 +447,12 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
         }
     }
 
+    /* This is fun, here's the order of operations:
+     * - if the Logind is running then use it
+     * - if UPower < 0.99.0 then use it (don't make changes on the user unless forced)
+     * - if ConsoleKit2 is running then use it
+     * - if everything else fails use our built-in fallback
+     */
     if ( LOGIND_RUNNING () )
     {
 	xfpm_systemd_sleep (power->priv->systemd, sleep_time, &error);
@@ -403,11 +471,25 @@ xfpm_power_sleep (XfpmPower *power, const gchar *sleep_time, gboolean force)
 #else
 	if (!g_strcmp0 (sleep_time, "Hibernate"))
         {
+	    if (check_for_consolekit2 (power))
+	    {
+		xfpm_console_kit_hibernate (power->priv->console, &error);
+	    }
+	    else
+	    {
                 xfpm_suspend_try_action (XFPM_HIBERNATE);
+	    }
         }
         else
         {
+	    if (check_for_consolekit2 (power))
+	    {
+		xfpm_console_kit_suspend (power->priv->console, &error);
+	    }
+	    else
+	    {
                 xfpm_suspend_try_action (XFPM_SUSPEND);
+	    }
         }
 #endif
     }
@@ -1180,36 +1262,36 @@ static void xfpm_power_get_property (GObject *object,
 
     switch (prop_id)
     {
-	case PROP_ON_BATTERY:
-	    g_value_set_boolean (value, power->priv->on_battery);
-	    break;
-	case PROP_AUTH_HIBERNATE:
-	    g_value_set_boolean (value, power->priv->auth_hibernate);
-	    break;
-	case PROP_AUTH_SUSPEND:
-	    g_value_set_boolean (value, power->priv->auth_suspend);
-	    break;
-	case PROP_CAN_SUSPEND:
-	    g_value_set_boolean (value, power->priv->can_suspend);
-	    break;
-	case PROP_CAN_HIBERNATE:
-	    g_value_set_boolean (value, power->priv->can_hibernate);
-	    break;
-	case PROP_HAS_LID:
-	    g_value_set_boolean (value, power->priv->lid_is_present);
-	    break;
-	case PROP_PRESENTATION_MODE:
-	    g_value_set_boolean (value, power->priv->presentation_mode);
-	    break;
+    case PROP_ON_BATTERY:
+        g_value_set_boolean (value, power->priv->on_battery);
+        break;
+    case PROP_AUTH_HIBERNATE:
+        g_value_set_boolean (value, power->priv->auth_hibernate);
+        break;
+    case PROP_AUTH_SUSPEND:
+        g_value_set_boolean (value, power->priv->auth_suspend);
+        break;
+    case PROP_CAN_SUSPEND:
+        g_value_set_boolean (value, power->priv->can_suspend);
+        break;
+    case PROP_CAN_HIBERNATE:
+        g_value_set_boolean (value, power->priv->can_hibernate);
+        break;
+    case PROP_HAS_LID:
+        g_value_set_boolean (value, power->priv->lid_is_present);
+        break;
+    case PROP_PRESENTATION_MODE:
+        g_value_set_boolean (value, power->priv->presentation_mode);
+        break;
     case PROP_ON_AC_BLANK:
-      g_value_set_int (value, power->priv->on_ac_blank);
-      break;
+        g_value_set_int (value, power->priv->on_ac_blank);
+        break;
     case PROP_ON_BATTERY_BLANK:
-      g_value_set_int (value, power->priv->on_battery_blank);
-      break;
-        default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-            break;
+        g_value_set_int (value, power->priv->on_battery_blank);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
     }
 }
 
@@ -1223,22 +1305,22 @@ static void xfpm_power_set_property (GObject *object,
 
     switch (prop_id)
     {
-	case PROP_PRESENTATION_MODE:
-	    xfpm_power_change_presentation_mode (power, g_value_get_boolean (value));
-	    break;
+    case PROP_PRESENTATION_MODE:
+        xfpm_power_change_presentation_mode (power, g_value_get_boolean (value));
+        break;
     case PROP_ON_AC_BLANK:
-          on_ac_blank = g_value_get_int (value);
-          power->priv->on_ac_blank = on_ac_blank;
-          xfpm_update_blank_time (power);
-          break;
-      case PROP_ON_BATTERY_BLANK:
-          on_battery_blank = g_value_get_int (value);
-          power->priv->on_battery_blank = on_battery_blank;
-          xfpm_update_blank_time (power);
-          break;
-        default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-            break;
+        on_ac_blank = g_value_get_int (value);
+        power->priv->on_ac_blank = on_ac_blank;
+        xfpm_update_blank_time (power);
+        break;
+    case PROP_ON_BATTERY_BLANK:
+        on_battery_blank = g_value_get_int (value);
+        power->priv->on_battery_blank = on_battery_blank;
+        xfpm_update_blank_time (power);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
     }
 }
 
@@ -1537,7 +1619,7 @@ static gboolean xfpm_power_dbus_reboot   (XfpmPower *power,
 static gboolean xfpm_power_dbus_hibernate (XfpmPower * power,
 					 GError **error)
 {
-    if ( !power->priv->auth_suspend )
+    if ( !power->priv->auth_hibernate )
     {
 	g_set_error (error, XFPM_ERROR, XFPM_ERROR_PERMISSION_DENIED,
                     _("Permission denied"));
